//
//
//
#ifndef IO_H
#define IO_H

#define MAX_SCAN_VAL    11
#define MAX_EXP_GOLOMB  16

//
//  Bitstream
//
typedef struct
{
  int             byte_pos;           //!< current position in bitstream;
  int             bits_to_go;         //!< current bitcounter
  byte            byte_buf;           //!< current buffer for last written byte
  int             stored_byte_pos;    //!< storage for position in bitstream;
  int             stored_bits_to_go;  //!< storage for bitcounter
  byte            stored_byte_buf;    //!< storage for buffer of last written byte

  byte            byte_buf_skip;      //!< current buffer for last written byte
  int             byte_pos_skip;      //!< storage for position in bitstream;
  int             bits_to_go_skip;    //!< storage for bitcounter

  byte            *streamBuffer;      //!< actual buffer for written bytes
  int             write_flag;         //!< Bitstream contains data and needs to be written
} Bitstream;

//
// Syntaxelement
//
typedef struct syntaxelement
{
  int                 type;           //!< type of syntax element for data part.
  int                 value1;         //!< numerical value of syntax element
  int                 value2;         //!< for blocked symbols, e.g. run/level
  int                 len;            //!< length of code
  int                 inf;            //!< info part of UVLC code
  unsigned int        bitpattern;     //!< UVLC bitpattern
  int                 context;        //!< CABAC context
  int                 k;              //!< CABAC context for coeff_count,uv
} SyntaxElement;


int read_one_new_frame(int height, int width, int skip, int start, int imgNo, FILE *p_in, imgStruct *img);
void write_image(int height, int width, FILE *p_out, imgStruct img);

Bitstream * allocBitstream(int size);
int freeBitstream(Bitstream * bitstream);
void saveBitstream(Bitstream *bitstream, char *fname);

int symbol2vlc(SyntaxElement *sym);
int u_1(char *tracestring, int value, Bitstream *bitstream);
int u_v(int n, char *tracestring, int value, Bitstream *bitstream);
int golombEncode(int coeff, int k, Bitstream *bitstream, char *tracestring);
int lengthGolomb(int coeffVal, int k);
int allOnes(int *v, int len);

int writePredFlags(int force0, int predMethod, int codedVarBins[NO_VAR_BINS], 
                   int filters_per_group, Bitstream *bitstream, int createBistream);
int writeFilterCodingParams(int minKStart, int maxScanVal, int kMinTab[], Bitstream *bitstream, int createBistream);
int writeFilterCoeffs(int sqrFiltLength, int filters_per_group, int pDepthInt[], 
                      int **FilterCoeff, int kMinTab[], Bitstream *bitstream, int createBitstream);
int codeAuxInfo(int filtNo, int noFilters, int varIndTab[NO_VAR_BINS], 
                int usePrevFiltBest[NO_VAR_BINS], int prevFiltInd, int frNo, Bitstream *bitstream, int createBitstream);
#endif // IO_H
